from Globals import InitializeClass
from Globals import Persistent
from AccessControl import ClassSecurityInfo, getSecurityManager
from types import StringType, ListType, TupleType

__version__ = '$Id: CollectorNGProperties.py,v 1.5 2003/02/10 13:54:39 ajung Exp $'


class CollectorNGPropertyManager(Persistent):
    """ a manager for CollectorNG properties similar to Zopes PropertyManager """

    security = ClassSecurityInfo()
    security.declareObjectPublic()

    meta_type = 'CollectorNGPropertymanager'

    def __init__(self, properties):
        self._properties = {}

        for p in properties:
            self.addProperty(p)

        self._p_changed = 1

    security.declarePublic('addProperty')
    def addProperty(self, prop):
        self._properties[ prop.getId()] = prop

    security.declarePublic('hasProperty')
    def hasProperty(self, id):
        return self._properties.has_key(id)    

    security.declarePublic('getPropertyIds')
    def getPropertyIds(self):
        return self._properties.keys()

    security.declarePublic('getPropertyById')
    def getPropertyById(self, id):
        return self._properties[id]

    security.declarePublic('getProperties')
    def getProperties(self, category=None, used=1):
        """ return properties and maybe filter by category and/or used flage """

        def __s(a,b): return cmp(a.getId(), b.getId())

        props =  self._properties.values()
        if category:
            props = [ prop  for prop in props if prop.getCategory()==category ]
        if used:
            props = [ prop  for prop in props if prop.getUsed()==1 ]

        props.sort(__s)
        return props


    security.declarePublic("getCategories")
    def getCategories(self):
        """ return sequence of categories """

        cats = []
        for prop in self._properties.values():
            if not prop.getCategory() in cats:
                cats.append(prop.getCategory())
        return cats


InitializeClass(CollectorNGPropertyManager)


class CollectorNGProperty(Persistent):

    security = ClassSecurityInfo()
    security.declareObjectPublic()

    def __init__(self, id, description, type, values='', default='', used=0, mandatory=0, help='', notify_on_change=None, permission=(), category=None):

        self._id = id
        self._type = type
        self._description = description
        self._values = values
        self._default = default
        self._used = used
        self._mandatory = mandatory
        self._help = help
        self._permission = permission
        self._category = category
        self._notify_on_change = notify_on_change
            

    security.declarePublic('getDefaultValue')
    def getDefaultValue(self):
        return self._default

    security.declarePublic('getValues')
    def getValues(self):
        return self._values

    security.declarePublic('getId')
    def getId(self):    
        """ return ID """
        return self._id

    security.declarePublic('getType')
    def getType(self):  
        """ return type """
        return self._type

    security.declarePublic('getDescription')
    def getDescription(self):  
        """ return description """
        return self._description

    security.declarePublic('getUsed')
    def getUsed(self):
        return self._used

    security.declarePublic('getMandatory')
    def getMandatory(self):
        return self._mandatory

    security.declarePublic('getHelp')
    def getHelp(self):
        return self._help

    security.declarePublic('getPermission')
    def getPermission(self):
        return self._permission

    security.declarePublic('getNotifyOnChange')
    def getNotifyOnChange(self):
        return self._notify_on_change    

    security.declarePublic('getCategory')
    def getCategory(self):
        return self._category

    security.declarePublic('setDefaultValue')
    def setDefaultValue(self, value):
        self._default = value

    security.declarePublic('setValues')
    def setValues(self, value):
        self._values = value

    security.declarePublic('setType')
    def setType(self, value):  
        """ return type """
        self._type = value

    security.declarePublic('setDescription')
    def setDescription(self, value):  
        """ return description """
        self._description = value

    security.declarePublic('setUsed')
    def setUsed(self, value):
        self._used  = value

    security.declarePublic('setMandatory')
    def setMandatory(self, value):
        self._mandatory = value

    security.declarePublic('setPermission')
    def setPermission(self, value):
        self._permission = permission

    security.declarePublic('setCategory')
    def setCategory(self, value):
        self._category = category

    security.declarePublic('setNotifyOnChange')
    def setNotifyOnChange(self, value):
        self._notify_on_change = value


    security.declarePublic('getRepresentation')
    def getRepresentation(self, value, key=0): 
        """ Values are either single keys or key-value pairs.
            For the given value this function either returns
            the value itself (single keys) or the corresponding
            key for the value (key-value pairs)
        """

        d = self.getValuesAsDict()
        try:
            return d[str(value)]
        except KeyError:
            return value


    security.declarePublic('getValuesAsList')
    def getValuesAsList(self):
        """ return values as list of (key,value) pairs """

        lst = []

        for item in self._values.split('|'):
            if item.find(',') > -1: k,v = item.split(',', 1)
            else:                   k = v = item                  
            lst.append( (k, v) )
    
        return lst

    security.declarePublic('getValuesAsDict')
    def getValuesAsDict(self):
        """ return values as dictionary  """

        dict = {}
        for k,v in self.getValuesAsList(): dict[k] = v
        return dict


InitializeClass(CollectorNGProperty)


