##############################################################################
#
# Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE
#
##############################################################################

"""This file is an installation script (External method) for the CMF CollectorNG.

To use, add an external method to the root of the CMF Site that you want CMF
CollectorNG registered in with the configuration:

 id: install_collectorNG
 title (optional): Install CollectorNG
 module name: CMFCollectorNG.Install
 function name: install

Then go to the management screen for the newly added external method and click
the 'Try it' tab.  The install function will execute and give information about
the steps it took to register and install the CMF CollectorNG into the CMF Site
instance. 
"""

from Products.CMFCore.TypesTool import ContentFactoryMetadata
from Products.CMFCore.DirectoryView import addDirectoryViews
from Products.CMFCore.utils import getToolByName
from Products import CMFCollectorNG
from cStringIO import StringIO
from Acquisition import aq_base


def install(self):
    """ Register the Collector with portal_types and friends """

    # add roles to CMF Site instance 

    assert self.meta_type == 'CMF Site'

    for role in ("TrackerAdmin", "Supporter", "Reporter"):
        if not role in self.valid_roles():
            self._addRole(role)

    out = StringIO()
    types_tool = getToolByName(self, 'portal_types')
    skins_tool = getToolByName(self, 'portal_skins')
    metadata_tool = getToolByName(self, 'portal_metadata')
    memberdata_tool = getToolByName(self, 'portal_memberdata')
    catalog = getToolByName(self, 'portal_catalog')

    # add some new properties to memberdatatool
    for key,default in (
               ('submitter_company', '') , ('submitter_position', '') , 
               ('submitter_name', '') , ('submitter_city', '') , 
               ('submitter_address', '') , ('submitter_fax', '') , 
               ('submitter_send_attachments','no'),
               ('submitter_phone', '') , ('collector_search_position', 'top')):
        try:
            memberdata_tool.manage_addProperty(key, default, 'string')
            out.write('Added "%s" to memberdata_tool\n' % key)
        except: 
            out.write('Skipping "%s" - already in memberdata_tool\n' % key)

    # Borrowed from CMFDefault.Portal.PortalGenerator.setupTypes()
    # We loop through anything defined in the factory type information
    # and configure it in the types tool if it doesn't already exist
    for t in CMFCollectorNG.factory_type_information:
        if t['id'] not in types_tool.objectIds():
            cfm = apply(ContentFactoryMetadata, (), t)
            types_tool._setObject(t['id'], cfm)
            out.write('Registered %s with the types tool\n' % t['id'])
        else:
            out.write('Skipping "%s" - already in types tool\n' % t['id'])

    # Setup the skins
    # This is borrowed from CMFDefault/scripts/addImagesToSkinPaths.pys
    if 'collectorng' not in skins_tool.objectIds():
        # We need to add Filesystem Directory Views for any directories
        # in our skins/ directory.  These directories should already be
        # configured.
        addDirectoryViews(skins_tool, 'skins', CMFCollectorNG.collector_globals)
        out.write("Added collectorng skin directory view to portal_skins\n")

    # Now we need to go through the skin configurations and insert
    # 'collector' into the configurations.  Preferably, this should be
    # right before where 'content' is placed.  Otherwise, we append
    # it to the end.
    skins = skins_tool.getSkinSelections()

    for skin in skins:
        path = skins_tool.getSkinPath(skin)
        path = [p.strip() for p in  path.split(',')]

        if 'collectorng' not in path:
            try: path.insert(0, 'collectorng')
            except ValueError:
                path.append('collectorng')
                
            path = ', '.join(path)
            # addSkinSelection will replace exissting skins as well.
            skins_tool.addSkinSelection(skin, path)
            out.write("Added 'collectorng' to %s skin\n" % skin)
        else:
            out.write("Skipping %s skin, 'collectorng' is already set up\n" % (
                skin))

#    for skin in skins:
#        path = skins_tool.getSkinPath(skin)
#        path = [p.strip() for p in  path.split(',')]
#
#        if 'collectorng1' not in path:
#            try: path.insert(0, 'collectorng1')
#            except ValueError:
#                path.append('collectorng1')
#                
#            path = ', '.join(path)
#            # addSkinSelection will replace exissting skins as well.
#            skins_tool.addSkinSelection(skin, path)
#            out.write("Added 'collectorng1' to %s skin\n" % skin)
#        else:
#            out.write("Skipping %s skin, 'collectorng1' is already set up\n" % (
#                skin))

    return out.getvalue()

