from transcript_formatter import format_transcript_as_xml
from AccessControl import ClassSecurityInfo 
from Products.CMFCore.CMFCorePermissions import View
from Globals import InitializeClass
from cStringIO import StringIO
from types import *

_mapping = {
    IntType: 'int',
    StringType: 'string',
    UnicodeType: 'string'
}

xml_header = '<?xml version="1.0" encoding="iso-8859-1"?>'

class xmlDoc:
    """ a dumb XML doc writer class """

    def __init__(self, basetag=None, id=None):
        self.doc = StringIO()
        self.basetag = basetag
        self.id = id

        self.add(xml_header)

        if self.basetag:
            if self.id:
                self.add('<%s id="%s">' % (self.basetag, self.id))
            else:
                self.add('<%s>' % self.basetag)

    def add(self, text):
        self.doc.write(text + '\n')

    def start_tag(self, tag):
        self.add('<%s>' % tag)

    def close_tag(self, tag):
        self.add('</%s>' % tag)

    def getContent(self):
        if self.basetag:
            self.add('</%s>' % self.basetag)
        return self.doc.getvalue()


class CollectorXML:
    """ mix-in class for CollectorNG instances """

    security = ClassSecurityInfo()

    security.declareProtected(View, 'getIssuesAsXML')
    def getIssuesAsXML(self):
        """ return sequence of all objectIds """

        doc = xmlDoc('issues')
        for id in  self.objectIds('CMF CollectorNG Issue'):
            doc.add('<issue id="%s"/>' % id)
        return doc.getContent()


    security.declareProtected(View, 'getIssuePropertiesAsXML')
    def getIssuePropertiesAsXML(self):
        """ return an XML representation of the ticket properties """

        doc = xmlDoc('trackerproperties', self.id)

        for p in self.issuePropertyManager.getProperties():

            doc.add('<property id="%s" default="%s" values="%s"'
                     ' type="%s" description="%s" used="%s" mandatory="%s"'
                     ' category="%s" />' % \
                     (p.getId(), p.getDefaultValue(), p.getValues(), 
                      p.getType(), p.getDescription(), p.getUsed(), 
                      p.getMandatory(), p.getCategory()))

        return doc.getContent()

InitializeClass(CollectorXML)

class IssueXML:        
    """ mix-in class for CollectorNGIssue instances """

    security = ClassSecurityInfo()

    def __check_permission(self):
        if not self.isVisible():
            raise RuntimeError, "no permission"

    def _getdata(self):
        """ return ticket metadata as dictionary """
            
        res = {}
        for prop in self.issuePropertyManager.getProperties():
            v = getattr(self, prop.getId())
            if callable(v): v = v()

            try:
                name = v.__class__.__name__
                if name=='DateTime': v = v.ISO()            
            except: pass
                
            res[prop.getId()]  = v
        
        return res


    security.declareProtected(View, 'asXML')
    def asXML(self, format="plain"):
        """ return ticket as XML """

        self.__checkpermission()

        if format=="plain":
            return self._getIssueAsXML_Plain()
        else:
            return self._getIssueAsXML_WDDX()


    def _getIssueAsXML_WDDX(self):
        """ return ticket metadata as XML WDDX"""

        doc = xmlDoc()

        doc.add('<header/>')
        doc.start_tag('data')
        doc.start_tag('recordset')

        for k,v in self._getdata().items():

            doc.add('<field name="%s">' % k)

            if isinstance(v, StringType):
                doc.add('<string>%s</string>' % v)                            
            elif isinstance(v, IntType) or isinstance(v, FloatType):
                doc.add('<number>%s</number>' % v)                            
            elif isinstance(v, ListType):
                for item in v:
                    doc.add('<string>%s</string>' %  item)                            
            else:
                raise ValueError, 'unknown type for %s' % k

            doc.add('</field>')

        doc.close_tag('recordset')
        doc.close_tag('data')

        return doc.getContent()


    def _getIssueAsXML_Plain(self):
        """ return ticket metadata as XML Plain"""

        doc = xmlDoc('ticket', self.id) 

        for k,v in self._getdata().items():

            if isinstance(v, StringType):
                doc.add('<%s type="string">%s</%s>' % (k, v, k))                            
            elif isinstance(v, IntType):
                doc.add('<%s type="int">%s</%s>' % (k, v, k))                            
            elif isinstance(v, FloatType):
                doc.add('<%s type="float">%s</%s>' % (k, v, k))                            
            elif isinstance(v, ListType):
                doc.add('<%s type="list">' % k)
                for item in v:
                    doc.add('<item type="%s">%s</item>' % (_mapping[type(item)], item))                            
                doc.add('</%s>' % k)
            else:
                raise ValueError, 'unknown type for %s' % k

        return doc.getContent()


    security.declareProtected(View, 'getTranscriptAsXML')
    def getTranscriptAsXML(self):
        """ return the tickets transcript as XML """

        self.__checkpermission()

        doc = xmlDoc('ticket', id) 
        doc.add( format_transcript_as_xml(self) )
        return doc.getContent()

InitializeClass(IssueXML)
