#!/usr/local/bin/python2.1

"""
Email filter for CMFCollectorNG for the submission of new
issues and followups.
"""

import sys, re, os, base64 
#sys.path.append('/local/tracker/instance/Products/CMFCollectorNG/')
import getopt, cStringIO, smtplib
import mimetypes, ConfigParser
import email, email.Iterators
from email.Header import decode_header 
from email.MIMEText import MIMEText
import httplib, urllib, time
import logging


# Logger
logging.fileConfig(os.path.join(os.path.expandvars("$HOME"), "logger.ini") )
L = logging.getLogger("collector")


# Read configuration from $HOME/.cmfcollectorngmail.ini
_config =  ".cmfcollectorngmail.ini"
CP = ConfigParser.ConfigParser()    
CP.read( (os.path.join(os.path.expandvars("$HOME"), _config), _config) )

TRACKER_HOST = CP.get("tracker","host")
TRACKER_URL  = CP.get("tracker","url")
TRACKER_USER = CP.get("tracker","user")
TRACKER_PASSWORD = CP.get("tracker","password")
TRACKER_NEW_ISSUE = CP.get("tracker", "new_issue")
TRACKER_FOLLOWUP = CP.get("tracker", "followups")
SMTP_HOST = CP.get("smtp","host")
SMTP_FROM = CP.get("smtp","from")
DEADLINE_DAYS = int(CP.get("misc","deadline_days"))
DEBUG = int(CP.get("misc","debug"))

deadline = time.strftime('%m-%d-%Y',time.localtime(time.time() + DEADLINE_DAYS*24*3600))

from_reg = re.compile("\<(.*)\>")
reg1 = re.compile('\[(\w+)\] #(\w+) (.*)')
reg2 = re.compile('\[(.*?)\] (.*)')
keyvalue_reg = re.compile("^##(\w+):(.*)")
cookie_reg = re.compile("\*\*\* \((.*?)\) \*\*\*")


class InvalidSubject(Exception): pass
class FollowupError(Exception): pass
class NewIssueError(Exception): pass

class Result:

    from_addr = ""
    from_real = ""
    reply_to = ""
    title = ""
    collector = ""
    issue_num = ""
    cookie = ""
    parameters = {}
    payload = ""
    ok = 0
    reason = ""
    data = ""


def usage():
    print "%s [options]" % os.path.basename(sys.argv[0])
    print
    print "Options:"
    print "\t-h|--help     This help"



def main():

    opts,args = getopt.getopt(sys.argv[1:], 'h', ('help',))

    for k,v in opts:
        if k in ('-h', '--help'): usage(); sys.exit(1)


    R = parse_mail()
    process_result(R)

def parse_mail():

    text = sys.stdin.read()
    msg = email.message_from_string(text)

    R = Result()

    for part in msg.walk():

        if part.has_key("From"):
            R.from_real, R.from_addr = email.Utils.parseaddr(part.get("From"))

        if part.has_key("Reply-To"):
            R.reply_to = email.Utils.parseaddr(part.get("Reply-To"))[1]


        if part.has_key("Subject"):
            subject = decode_header(part.get("Subject"))[0][0]

            # Check for a followup
            mo = reg1.search(subject) 
            if mo: 
                R.collector = mo.group(1)
                R.issue_num = mo.group(2)
                R.title = mo.group(3)
            else:
                mo = reg2.search(subject) # new issue
                if mo:
                    R.collector = mo.group(1)
                    R.title = mo.group(2)

        # don't process HTML or similiar junk

        if not  part.get_content_type() in ('text/plain',):
            continue

        R.payload = ["Submitted by %s (%s)" % (R.from_real, R.from_addr), ""]
        R.parameters = {}

        for line in part.get_payload(decode=1).splitlines():
            line = line.strip()

            # check for separator and cookie
            if line.find("***") > -1:
                mo = cookie_reg.search(line)
                if mo:
                    R.cookie = mo.group(1)
                break

            else:
                mo = keyvalue_reg.match(line)
                if mo:
                    R.parameters[mo.group(1).lower()] = mo.group(2).strip()
                else:
                    R.payload.append(line)

        R.payload = "\n".join(R.payload)


    if DEBUG:
        L.debug('-' *78)
        L.debug('Email: %s (%s)' % (R.from_addr, R.from_real))
        L.debug('Title: %s' % R.title)
        L.debug('Tracker: %s' %  R.collector)
        L.debug('Issuenum: %s' % R.issue_num)
        L.debug('Cookie: %s' % R.cookie)
        L.debug('Parameters: %s' % R.parameters)
        L.debug('Payload:\n%s\n' % R.payload)

    return R

def process_result(R):

    if R.issue_num=="" and R.cookie=="":
        try:
            new_issue(R)
            send_confirmation(R, ok=1)
            L.info("Posting ok")
        except NewIssueError, reason:
            send_confirmation(R, ok=0)
            L.error("Posting failed")

    else:   
        try:
            followup(R)
            send_confirmation(R, ok=1)
            L.info("Posting ok")
        except FollowupError, reason:
            send_confirmation(R, ok=0)
            L.error("Posting failed")



def followup(R):

    params = urllib.urlencode( {
        'comment' : R.payload,
        'properties.submitter_name:record' : R.from_real,
        'properties.submitter_email:record' : R.from_addr,
        'email_submission.cookie:record:string' : R.cookie,
        })

    headers = { 'Content-Type' : 'application/x-www-form-urlencoded',
                'Accept' : 'text/plain',
                'Authorization': 'Basic ' + base64.encodestring('%s:%s' % (TRACKER_USER, TRACKER_PASSWORD))[:-1],
              }

    url = "%s/%s/%s/%s" % (TRACKER_URL, R.collector, R.issue_num, TRACKER_FOLLOWUP)
    L.debug("URL called: %s" % url)
    conn = httplib.HTTPConnection(TRACKER_HOST)
    conn.request("POST",  url, params, headers)
    response = conn.getresponse()
    data = response.read()   
    status = response.status
    reason = response.reason
    response.close()
 
    L.debug("Status: %s" % status)
    L.debug("Reason: %s" % reason)
    L.debug("Data: %s" % data)

    if response.status  != 200:
        raise FollowupError, response.data
    


def new_issue(R):

    params = urllib.urlencode( {
        'properties.description:record' : R.payload,
        'properties.title:record' : R.title,
        'properties.topic:record' : R.parameters.get('topic', 'n/a') ,
        'properties.security_related:record:int' : 0,
        'properties.subtopic:record' :  R.parameters.get('subtopic', 'n/a'),
        'properties.submitter_id:record' : "myid",
        'properties.submitter_name:record' : R.from_real,
        'properties.submitter_email:record' : R.from_addr,
        'properties.operating_system:record' : "all",
        'properties.deadline:record:date' : deadline, 
        'properties.assignees:record:list' : '',
        'email_submission.token:record:string' : R.parameters.get('token','xxxxxxxxx') ,
        'email_submission.from_addr:record:string' : R.from_addr,
        'email_submission.reply_to:record:string' : R.reply_to,
        })

    L.info(params)   
    headers = { 'Content-Type' : 'application/x-www-form-urlencoded',
                'Accept' : 'text/plain',
                'Authorization': 'Basic ' + base64.encodestring('%s:%s' % (TRACKER_USER, TRACKER_PASSWORD))[:-1],
              }

    url = "%s/%s/%s" % (TRACKER_URL, R.collector, TRACKER_NEW_ISSUE)
    L.debug("URL called: %s" % url)
    conn = httplib.HTTPConnection(TRACKER_HOST)
    conn.request("POST",  url, params, headers)
    response = conn.getresponse()
    data = response.read()   
    status = response.status
    reason = response.reason
    response.close()

    L.debug("Status: %s" % status)
    L.debug("Reason: %s" % reason)
    L.debug("Data: %s" % data)

    if status in (400, 404):       # wrong URL, not found 
        R.reason = "Unknown tracker '%s'" % R.collector         
        raise NewIssueError, R.reason 

    if status  != 200:
        R.reason = data  
        raise NewIssueError, R.reason
    

def send_confirmation(R, ok=0):

    IO = cStringIO.StringIO()

    if ok: print >>IO,'Your posting was submitted successfully\n'
    else:  print >>IO,'Reason for the failure: %s' % R.reason

    print >>IO,'------------------ Original posting -------------------\n'
    print >>IO,R.payload
    
    print >>IO,"\n"
    print >>IO,'-'*78 
    print >>IO,"Please do not reply this automatically generated email"

    msg = MIMEText(IO.getvalue()
)
    msg["From"] = SMTP_FROM
    msg["To"] = R.from_addr
    msg["Message-ID"] = email.Utils.make_msgid()
    msg["Content-Type"] = 'text/plain; charset=iso-8859-1'

    if ok==1:
        msg['Subject'] = '[%s] Status of your posting: OK' % R.collector
    else: 
        msg['Subject'] = '[%s] Status of your posting: FAILED' % R.collector


    server = smtplib.SMTP(SMTP_HOST)
    server.set_debuglevel(0)
    server.sendmail(SMTP_FROM, [R.from_addr], msg.as_string())
    server.quit()

if __name__ == '__main__':

    try:
        main()
    except:
        L.fatal("execution failed", exc_info=sys.exc_info())

