##############################################################################
#
# Copyright (c) 2001 Zope Corporation and Contributors. All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.0 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE
#
##############################################################################

"""Sundry collector utilities."""

def users_for_local_role(object, userids, role):
    """Give only designated userids specified local role.

    Return 1 iff any role changes happened."""
    already = []
    changed = 0
    for u in object.users_with_local_role(role):
        if u in userids:
            already.append(u)
        else:
            changed = 1
            remove_local_role(object, u, role)
    for u in userids:
        if u not in already:
            changed = 1
            add_local_role(object, u, role)
    return changed

def add_local_role(object, userid, role):
    """Add object role for userid if not already there."""
    roles = list(object.get_local_roles_for_userid(userid))
    if role not in roles:
        roles.append(role)
        object.manage_setLocalRoles(userid, roles)

def remove_local_role(object, userid, role):
    """Add object role for userid if not already there."""
    roles = list(object.get_local_roles_for_userid(userid))
    roles.remove(role)
    if roles:
        object.manage_setLocalRoles(userid, roles)
    else:
        object.manage_delLocalRoles([userid])

def get_email_fullname(self, userid):
    """Get full_name or userid, and email, from membership tool."""

    from Products.CMFCore.utils import getToolByName

    mbrtool = getToolByName(self, 'portal_membership')
    user = mbrtool.getMemberById(userid)
    if user is not None:
        email = safeGetProperty(user, 'email', None)
        name = safeGetProperty(user, 'full_name', str(user))
        if '.' in name or ',' in name:
            name = '"%s"' % name
        return (name, email)
    return (None, None)

def safeGetProperty(userobj, property, default=None):
    """Defaulting user.getProperty(), allowing for variant user folders."""
    try:
        if not hasattr(userobj, 'getProperty'):
            return getattr(userobj, property, default)
        else:
            return userobj.getProperty(property, default)
    except:
        # We can't just itemize the possible candidate exceptions because one
        # is a string with spaces, which isn't interned and hence object id
        # won't match.  Sigh.
        import sys
        exc = sys.exc_info()[0]
        if (exc == 'Property not found'
            or isinstance(exc, TypeError)
            or isinstance(exc, AttributeError)
            or isinstance(exc, LookupError)):
            try:
                # Some (eg, our old LDAP user folder) support getProperty but
                # not defaulting:
                return userobj.getProperty(property)
            except:
                return default
        else:
            raise


def isValidEmailAddress(email):

    at = email.find('@')
    pt = email.find('.')
    # Minimal lenght must be 8
    if len(email) <= 7 :
        return 0
    # We must have an @ but not first and not last
    if at == -1 or at == 0 or email[-1] == '@':
        return 0
    # Same applies for the point
    if pt == -1 or pt == 0 or email[-1] == '.':
        return 0
    # Only one @ is valid
    if email.find('@',at+1) > -1:
        return 0
    # @ and . cannot be together
    if email[at-1] == '.' or email[at+1] == '.':
        return 0
    # subdomain must have at least 2 letters
    if email[at+2] == '.':
        return 0
    # no spaces allowed
    if email.find(' ') > -1:
        return 0
    return 1


def remove_dupes(lst):
    """ remove dupes from a list """

    ret = []
    for l in lst:
        if not l in ret:
            ret.append(l)

    return ret 


def lists_eq(l1, l2):
    """ compare two lists l1 and l2. l1 and l2 are equal if they
        contain the same elements. 
    """

    if not l1: l1 = []
    if not l2: l2 = []

    l1 = list(l1); l1.sort()
    l2 = list(l2); l2.sort()

    return l1==l2
